@@ -1,21 +1,26 @@
-use clap::{crate_version, value_t, values_t, App, Arg};
-use log::LevelFilter;
 use std::{
-    path::{PathBuf, MAIN_SEPARATOR},
+    env,
+    ffi::OsString,
+    fs::File,
+    io::{BufRead, BufReader},
+    path::{Path, PathBuf, MAIN_SEPARATOR},
     time::Duration,
 };
 
+use clap::{crate_version, value_t, values_t, App, Arg};
+use color_eyre::eyre::{Context, Report, Result};
+use log::LevelFilter;
 use watchexec::{
     config::{Config, ConfigBuilder},
-    error,
     run::OnBusyUpdate,
     Shell,
 };
 
-pub fn get_args() -> error::Result<(Config, LevelFilter)> {
+pub fn get_args() -> Result<(Config, LevelFilter)> {
     let app = App::new("watchexec")
         .version(crate_version!())
         .about("Execute commands when watched files change")
+        .after_help("Use @argfile as first argument to load arguments from the file `argfile` (one argument per line) which will be inserted in place of the @argfile (further arguments on the CLI will override or add onto those in the file).")
         .arg(Arg::with_name("command")
                  .help("Command to execute")
                  .multiple(true)
@@ -57,7 +62,7 @@ pub fn get_args() -> error::Result<(Config, LevelFilter)> {
                  .short("k")
                  .long("kill"))
         .arg(Arg::with_name("debounce")
-                 .help("Set the timeout between detected change and command execution, defaults to 150ms")
+                 .help("Set the timeout between detected change and command execution, defaults to 100ms")
                  .takes_value(true)
                  .value_name("milliseconds")
                  .short("d")
@@ -121,17 +126,39 @@ pub fn get_args() -> error::Result<(Config, LevelFilter)> {
         .arg(Arg::with_name("no-environment")
                  .help("Do not set WATCHEXEC_*_PATH environment variables for the command")
                  .long("no-environment"))
+        .arg(Arg::with_name("no-process-group")
+                 .help("Do not use a process group when running the command")
+                 .long("no-process-group"))
         .arg(Arg::with_name("once").short("1").hidden(true))
         .arg(Arg::with_name("watch-when-idle")
                  .help("Deprecated alias for --on-busy-update=do-nothing, which will become the default in 2.0.")
                  .short("W")
                  .long("watch-when-idle"));
 
-    let args = app.get_matches();
+    let mut raw_args: Vec<OsString> = env::args_os().collect();
+
+    if let Some(first) = raw_args.get(1).and_then(|s| s.to_str()) {
+        if let Some(arg_path) = first.strip_prefix('@').map(Path::new) {
+            let arg_file = BufReader::new(
+                File::open(arg_path)
+                    .wrap_err_with(|| format!("Failed to open argument file {:?}", arg_path))?,
+            );
+
+            let mut more_args: Vec<OsString> = arg_file
+                .lines()
+                .map(|l| l.map(OsString::from).map_err(Report::from))
+                .collect::<Result<_>>()?;
+
+            more_args.insert(0, raw_args.remove(0));
+            more_args.extend(raw_args.into_iter().skip(1));
+            raw_args = more_args;
+        }
+    }
+
+    let args = app.get_matches_from(raw_args);
     let mut builder = ConfigBuilder::default();
 
-    let cmd: Vec<String> =
-        values_t!(args.values_of("command"), String).map_err(|err| err.to_string())?;
+    let cmd: Vec<String> = values_t!(args.values_of("command"), String)?;
     builder.cmd(cmd);
 
     let paths: Vec<PathBuf> = values_t!(args.values_of("path"), String)
@@ -239,8 +266,9 @@ pub fn get_args() -> error::Result<(Config, LevelFilter)> {
     builder.no_vcs_ignore(args.is_present("no-vcs-ignore"));
     builder.no_ignore(args.is_present("no-ignore"));
     builder.poll(args.occurrences_of("poll") > 0);
+    builder.use_process_group(!args.is_present("no-process-group"));
 
-    let mut config = builder.build().map_err(|err| err.to_string())?;
+    let mut config = builder.build()?;
     if args.is_present("once") {
         config.once = true;
     }
